/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is Compiere ERP & CRM Smart Business Solution. The Initial
 * Developer of the Original Code is Jorg Janke. Portions created by Jorg Janke
 * are Copyright (C) 1999-2005 Jorg Janke.
 * All parts are Copyright (C) 1999-2005 ComPiere, Inc.  All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/
package org.compiere.grid;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import java.util.logging.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import org.compiere.apps.*;
import org.compiere.grid.ed.*;
import org.compiere.grid.tree.*;
import org.compiere.model.*;
import org.compiere.swing.*;
import org.compiere.util.*;

/**
 *  The Grid Controller is the panel for single and multi-row presentation
 *  and links to the Model Tab.
 *
 *  <pre>
 *  UI Structure:
 *  this    (BorderLayout)
 *      splitPane (JSplitPane)
 *          left
 *              graphicPanel
 *          right
 *              cardPanel   JPanel  (CardLayout)
 *                  srPane  JSplitPane
 * 						vPane	JScrollPane
 *              	        vPanel  VPanel (GridBagLayout)
 * 						vIncludedGC	GridController
 *                  mrPane  JScrollPane
 *                      vTable  VTable
 *
 *  <B>DataBinding:<B>
 *  - MultiRow - is automatic between VTable and MTable
 *  - SingleRow
 *		- from VEditors via fireVetoableChange(m_columnName, null, getText());
 *			(vetoableChange)
 *		- to VEditors via updateSingleRow -> Editor.setValue(object)
 *
 *  Event Chains
 *  -- Navigation --
 *  (VTable selection -> GridController.valueChanged)
 *  (APanel selection)
 *      + MTab.navivate
 *          + MTab.setCurrentRow
 *              + Update all MFields
 *                  + MField.setValue
 *                      + setContext
 *                      + fire PropertyChange "Value"
 *                          + VEditor.propertyChange
 *                              + VEditor.setValue
 *              + MTab.fireProperyChange "CurrentRow"
 *                  + VTable.propertyChange (setRowSelectionInterval)
 *                      + GridController.valueChange
 *                          + GridController.dynamicDisplay(complete)
 *              + MTab.fireDataStatusChanged
 *                  + APanel.statusChanged
 *
 *  -- ValueChanges --
 *  VEditor.fireVetoableChange
 *      + (VCellEditor.vetoableChange/getCellEditorValue)   -- multi-row source
 *      + (GridController.vetoableChange)                   -- single-row source
 *          + MTable.setValueAt
 *              + MField.setValue
 *                  + setContext
 *                  + fire PropertyChange "Value"
 *                      + VEditor.setValue
 *              + MTable.fireDataStatusChanged
 *                  + MTab.dataStatusChanged
 *                      + MTab.fireDataStatusChanged
 *                          + APanel.statusChanged
 *                  + GridController.dataStatusChanged
 *                      + GridController.dynamicDisplay(selective)
 *  </pre>
 * @author  Jirimuto
 * @version $Id: ACGridController.java,v 1.2 2006/10/09 11:38:48 jrmt Exp $
 */
public class ACGridController extends CPanel
	implements DataStatusListener, ListSelectionListener, 
		VetoableChangeListener,	PropertyChangeListener, MouseListener, ActionListener
{
	/**
	 *  Constructor - you need to call initGrid for instanciation
	 */
	public ACGridController()
	{
		try
		{
			jbInit();
		}
		catch(Exception e)
		{
			log.log(Level.SEVERE, "", e);
		}
	}   //  GridController

	/**
	 *  toString
	 *  @return string representation
	 */
	public String toString()
	{
		return "GridController for " + m_mTab;
	}   //  toString

	/**	Logger			*/
	private static CLogger log = CLogger.getCLogger(ACGridController.class);
	
	/**
	 *  The Layout
	 */
	private BorderLayout mainLayout = new BorderLayout();
	private CPanel toolbarPanel = new CPanel();
	private BorderLayout toolbarLayout = new BorderLayout();
	private JSplitPane splitPane = new JSplitPane();
	private CPanel graphPanel = new CPanel();
	private BorderLayout graphLayout = new BorderLayout();
	private CPanel cardPanel = new CPanel();
	private CardLayout cardLayout = new CardLayout();
	private JSplitPane srPane = new JSplitPane();
	private JScrollPane vPane = new JScrollPane();
	private ACGridController vIncludedGC = null;
	private CScrollPane mrPane = new CScrollPane();
	private CPanel xPanel = new CPanel();
	private FlowLayout xLayout = new FlowLayout();
	private VTable vTable = new VTable();
	private VACPanel vPanel = new VACPanel();
	
	private CButton bIgnore = new CButton();
	private CLabel bSeperatorNew = new CLabel();
	private CButton bNew = new CButton();
	private CButton bCopy = new CButton();
	private CButton bDelete = new CButton();
	private CButton bSave = new CButton();
	private CLabel bSeperatorRefresh = new CLabel();
	private CButton bRefresh = new CButton();
	
	/**
	 *  Static Layout init
	 *  @throws Exception
	 */
	private void jbInit() throws Exception
	{
		this.setLayout(mainLayout);
		this.add(toolbarPanel,BorderLayout.NORTH);
		toolbarPanel.setLayout( toolbarLayout );
		
		this.add(splitPane, BorderLayout.CENTER);
		splitPane.setOpaque(false);
		graphPanel.setLayout(graphLayout);
		//
		splitPane.add(graphPanel, JSplitPane.LEFT);
		splitPane.add(cardPanel, JSplitPane.RIGHT);
		splitPane.setBorder(null);
		splitPane.setName("gc_splitPane");
		splitPane.setDividerSize( 0 );
		//
		cardPanel.setLayout(cardLayout);
		cardPanel.add(srPane, "srPane");	//	Sequence Important!
		cardPanel.add(mrPane, "mrPane");
		cardPanel.setBorder(null);
		cardPanel.setName("gc_cardPanel");

		//  single row (w/o xPane it would be centered)
		srPane.setBorder(null);
		srPane.setName("gc_srPane");
		srPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
		srPane.add(vPane, JSplitPane.TOP);
		srPane.setTopComponent(vPane);
		srPane.setBottomComponent(null);	//	otherwise a button is created/displayed
		//
		vPane.getViewport().add(xPanel, null);
		xPanel.add(vPanel);
		vPane.setBorder(null);
		xPanel.setLayout(xLayout);
		xPanel.setName("gc_xPanel");
		xLayout.setAlignment(FlowLayout.LEFT);
		xLayout.setHgap(0);
		xLayout.setVgap(0);
		//  multi-row
		// mrPane.setBorder(null);
		mrPane.setBorder(null);
		mrPane.getViewport().add(vTable, null);
		mrPane.setName("gc_mrPane");
		//
		graphPanel.setBorder(null);
		graphPanel.setName("gc_graphPanel");
		splitPane.setDividerLocation(0);
		srPane.setDividerLocation(200);
		
	}   //  jbInit

	/**
	 *  Displose
	 */
	public void dispose()
	{
		log.config( "(" + m_mTab.toString() + ")");
		//  clear info
		stopEditor(false);
		if (m_mTab.needSave(true, false))
			m_mTab.dataIgnore();
		vIncludedGC = null;

		//  Listeners
		m_mTab.getTableModel().removeDataStatusListener(this);
		m_mTab.getTableModel().removeVetoableChangeListener(this);
		vTable.getSelectionModel().removeListSelectionListener(this);
		m_mTab.removePropertyChangeListener(vTable);

		//  editors
		Component[] comp = vPanel.getComponents();
		for (int i = 0; i < comp.length; i++)
		{
			if (comp[i] instanceof VEditor)
			{
				VEditor vEditor = (VEditor)comp[i];
				vEditor.removeVetoableChangeListener(this);
				String columnName = comp[i].getName();
				GridField mField = m_mTab.getField(columnName);
				if (mField != null)
					mField.removePropertyChangeListener(vEditor);
				vEditor.dispose();
			}
			
		}
		/** @todo Remove APanel Button listeners */

		vTable.removeAll();
		vTable.setModel(new DefaultTableModel());   //  remove reference
		vTable = null;
		vPanel.removeAll();
		vPanel = null;
		srPane.removeAll();
		srPane = null;
		splitPane.removeAll();
		splitPane = null;
		m_mTab = null;
		m_tree = null;
		this.removeAll();
	}   //  dispose

	/** Model Tab                   */
	private GridTab        m_mTab = null;

	/** Model included Tab          */
	private ArrayList<GridTab>      m_includedTabs = new ArrayList<GridTab>();
	/** Window                      */
	private int         m_WindowNo;
	/** Only Multi-Row exist        */
	private boolean     m_onlyMultiRow = false;
	/** Single/Multi Row indicator  */
	private boolean     m_singleRow = true;
	/** Veto Active                 */
	private boolean     m_vetoActive = false;
	
	/** Veto Active                 */
	private boolean     dataStatusChangeAcvtive = false;
	
	/** Tree Panel (optional)       */
	private VTreePanel  m_tree;
	
	public boolean initToolBar (GridTab mTab, ACPanel aPanel, MWindow mWindow)
	{
		VStackPanel vStackPanel = new VStackPanel();
		toolbarPanel.add( vStackPanel, BorderLayout.CENTER);
		ArrayList<Component> toolbarComponentList =  new ArrayList<Component>();
		Dimension toolbarDimension = new Dimension( 20, 20 );
		Dimension seperatorDimension = new Dimension( 5, 20 );
		
		bIgnore.setIcon(Env.getImageIcon("Ignore16.gif"));
		bIgnore.setToolTipText(Msg.getMsg(Env.getCtx(), "Ignore"));
		bIgnore.addActionListener(this);
		bIgnore.setSize( toolbarDimension );
		bIgnore.setPreferredSize( toolbarDimension );
		toolbarComponentList.add( bIgnore );
		
		bSeperatorNew.setText( " " );
		bSeperatorNew.setSize( seperatorDimension );
		bSeperatorNew.setPreferredSize( seperatorDimension );
		toolbarComponentList.add( bSeperatorNew );
		
		bNew.setIcon(Env.getImageIcon("New16.gif"));
		bNew.setToolTipText(Msg.getMsg(Env.getCtx(), "New"));
		bNew.addActionListener(this);
		bNew.setSize( toolbarDimension );
		bNew.setPreferredSize( toolbarDimension );
		toolbarComponentList.add( bNew );
		
		bCopy.setIcon(Env.getImageIcon("Copy16.gif"));
		bCopy.setToolTipText(Msg.getMsg(Env.getCtx(), "Copy"));
		bCopy.addActionListener(this);
		bCopy.setSize( toolbarDimension );
		bCopy.setPreferredSize( toolbarDimension );
		toolbarComponentList.add( bCopy );
		
		bDelete.setIcon(Env.getImageIcon("Delete16.gif"));
		bDelete.setToolTipText(Msg.getMsg(Env.getCtx(), "Delete"));
		bDelete.addActionListener(this);
		bDelete.setSize( toolbarDimension );
		bDelete.setPreferredSize( toolbarDimension );
		toolbarComponentList.add( bDelete );
		
		bSave.setIcon(Env.getImageIcon("Save16.gif"));
		bSave.setToolTipText(Msg.getMsg(Env.getCtx(), "Save"));
		bSave.addActionListener(this);
		bSave.setSize( toolbarDimension );
		bSave.setPreferredSize( toolbarDimension );
		toolbarComponentList.add( bSave );
		
		bSeperatorRefresh.setText( " " );
		bSeperatorRefresh.setSize( seperatorDimension );
		bSeperatorRefresh.setPreferredSize( seperatorDimension );
		toolbarComponentList.add( bSeperatorRefresh );
		
		bRefresh.setIcon(Env.getImageIcon("Refresh16.gif"));
		bRefresh.setToolTipText(Msg.getMsg(Env.getCtx(), "Refresh"));
		bRefresh.addActionListener(this);
		bRefresh.setSize( toolbarDimension );
		bRefresh.setPreferredSize( toolbarDimension );
		toolbarComponentList.add( bRefresh );
		
		
		vStackPanel.setComponentList( toolbarComponentList );
		
		return true;
	}

	/**************************************************************************
	 *  Init Grid.
	 *  <pre>
	 *  - Map table to model
	 *  - Update (multi-row) table info with renderers/editors
	 *  - build single-row panel
	 *  - initialize display
	 *  </pre>
	 *  @param mTab tab
	 *  @param onlyMultiRow only table
	 *  @param WindowNo window no
	 *  @param aPanel optional Application Panel for adding button listeners
	 * 	@param mWindow parent Window Model
	 *  @return true if initialized
	 */
	public boolean initGrid (GridTab mTab, boolean onlyMultiRow,  
		int WindowNo, ACPanel aPanel, MWindow mWindow)
	{
		log.config( "(" + mTab.toString() + ")");
		m_mTab = mTab;
		m_WindowNo = WindowNo;
		m_onlyMultiRow = onlyMultiRow;
		setName("GC-" + mTab);

		//  Set up Multi Row Table
		vTable.setModel(m_mTab.getTableModel());
		
		//  Update Table Info -------------------------------------------------
		int size = setupVTable (aPanel, m_mTab, vTable);

		//  Set Color on Tab Level
		//  this.setBackgroundColor (mTab.getColor());

		//  Single Row  -------------------------------------------------------
		if (!m_onlyMultiRow)
		{
			//	Set Softcoded Mnemonic &x
			for (int i = 0; i < size; i++)
			{
				GridField mField = m_mTab.getField(i);
				if (mField.isDisplayed())
					vPanel.setMnemonic(mField);
			}   //  for all fields
			
			//	Add Fields
			for (int i = 0; i < size; i++)
			{
				GridField mField = m_mTab.getField(i);
				if (mField.isDisplayed())
				{
					VEditor vEditor = VEditorFactory.getEditor(m_mTab, mField, false);
					if (vEditor == null)
					{
						log.severe("Editor not created for " + mField.getColumnName());
						continue;
					}
					//  MField => VEditor - New Field value to be updated to editor
					mField.addPropertyChangeListener(vEditor);
					//  VEditor => this - New Editor value to be updated here (MTable)
					vEditor.addVetoableChangeListener(this);

					//  Add to VPanel
					vPanel.addField(vEditor, mField);
					//  APanel Listen to buttons
					if (mField.getDisplayType() == DisplayType.Button && aPanel != null)
						((JButton)vEditor).addActionListener (aPanel);
					
				}
				
			}   //  for all fields

			//	No Included Grid Controller
			srPane.setResizeWeight(1);	//	top part gets all
			srPane.setDividerSize (0);
			srPane.setDividerLocation (9999);

			//  Use SR to size MR
			mrPane.setPreferredSize(vPanel.getPreferredSize());
		}   //  Single-Row

		//  Tree Graphics Layout
		int AD_Tree_ID = 0;
		if (m_mTab.isTreeTab())
			AD_Tree_ID = MTree.getDefaultAD_Tree_ID (
				Env.getAD_Client_ID(Env.getCtx()), m_mTab.getKeyColumnName());
		if (m_mTab.isTreeTab() && AD_Tree_ID != 0)
		{
			m_tree = new VTreePanel(m_WindowNo, false, true);
			if (m_mTab.getTabNo() == 0)	//	initialize other tabs later
				m_tree.initTree(AD_Tree_ID);
			m_tree.addPropertyChangeListener(VTreePanel.NODE_SELECTION, this);
			graphPanel.add(m_tree, BorderLayout.CENTER);
			splitPane.setDividerLocation(250);
		//	splitPane.resetToPreferredSizes();
		}
		else    //  No Graphics - hide
		{
			graphPanel.setPreferredSize(new Dimension(0,0));
			splitPane.setDividerSize(0);
			splitPane.setDividerLocation(0);
		}

		//  Receive DataStatusChanged info from MTab
		m_mTab.addDataStatusListener(this);
		//  Receive vetoableChange info from MTable when saving
		m_mTab.getTableModel().addVetoableChangeListener(this);
		//	Selection Listener -> valueChanged
		vTable.getSelectionModel().addListSelectionListener(this);
		//  Navigation (RowChanged)
		m_mTab.addPropertyChangeListener(vTable);

		//  Update UI
		vTable.autoSize(true);

		setTabLevel(m_mTab.getTabLevel());

		//  Set initial presentation
		if (onlyMultiRow || !m_mTab.isSingleRow())
			switchMultiRow();
		else
			switchSingleRow();
	//	log.config( "GridController.dynInit (" + mTab.toString() + ") - fini");
		return true;
		
	}   //  initGrid

	/**************************************************************************
	 *  Init Grid.
	 *  <pre>
	 *  - Map table to model
	 *  - Update (multi-row) table info with renderers/editors
	 *  - build single-row panel
	 *  - initialize display
	 *  </pre>
	 *  @param mTab tab
	 *  @param onlyMultiRow only table
	 *  @param WindowNo window no
	 *  @param aPanel optional Application Panel for adding button listeners
	 * 	@param mWindow parent Window Model
	 *  @return true if initialized
	 */
	public boolean initGridWithFormat (GridTab mTab, M_TabFormat m_TabFormat, boolean onlyMultiRow,  
		int WindowNo, ACPanel aPanel, MWindow mWindow )
	{
		log.config( "(" + mTab.toString() + ")");
		m_mTab = mTab;
		m_WindowNo = WindowNo;
		m_onlyMultiRow = onlyMultiRow;
		setName("GC-" + mTab);

		//  Set up Multi Row Table
		vTable.setModel(m_mTab.getTableModel());

		//  Update Table Info -------------------------------------------------
		int size = setupVTable (aPanel, m_mTab, vTable);
		
		if( m_TabFormat != null ) {
			vPanel.setRowAndCol( m_TabFormat.getTab_RowCount(), m_TabFormat.getTab_ColCount() );
		}
		
		//  Single Row  -------------------------------------------------------
		if (!m_onlyMultiRow)
		{
			//	Set Softcoded Mnemonic &x
			for (int i = 0; i < size; i++)
			{
				GridField mField = m_mTab.getField(i);
				if (mField.isDisplayed())
					vPanel.setMnemonic(mField);
			}   //  for all fields
			
			//	Add Fields
			for (int i = 0; i < size; i++)
			{
				GridField mField = m_mTab.getField(i);
				if (mField.isDisplayed())
				{
					VEditor vEditor = VEditorFactory.getEditor(m_mTab, mField, false);
					if (vEditor == null)
					{
						log.severe("Editor not created for " + mField.getColumnName());
						continue;
					}
					//  MField => VEditor - New Field value to be updated to editor
					mField.addPropertyChangeListener(vEditor);
					//  VEditor => this - New Editor value to be updated here (MTable)
					vEditor.addVetoableChangeListener(this);
					
					M_FieldFormat m_fieldFormat = M_FieldFormat.getM_FieldFormat(Env.getCtx(), m_mTab.getAD_Tab_ID(), mField.getAD_Column_ID() );
					//  Add to VPanel
					vPanel.addFieldWithFormat(vEditor, mField, m_fieldFormat);
					
					//  APanel Listen to buttons
					if (mField.getDisplayType() == DisplayType.Button && aPanel != null)
						((JButton)vEditor).addActionListener (aPanel);
				}
				
			}   //  for all fields
			
			if( m_TabFormat!=null ){
				vPanel.prepareComplete( m_TabFormat.getTab_ColCount() );
			} else {
				vPanel.prepareComplete( 0 );
			}
			
			//	No Included Grid Controller
			srPane.setResizeWeight(1);	//	top part gets all
			srPane.setDividerSize (0);
			srPane.setDividerLocation (9999);

			//  Use SR to size MR
			mrPane.setPreferredSize(vPanel.getPreferredSize());
		}   //  Single-Row

		//  Tree Graphics Layout
		int AD_Tree_ID = 0;
		if (m_mTab.isTreeTab())
			AD_Tree_ID = MTree.getDefaultAD_Tree_ID (
				Env.getAD_Client_ID(Env.getCtx()), m_mTab.getKeyColumnName());
		if (m_mTab.isTreeTab() && AD_Tree_ID != 0)
		{
			m_tree = new VTreePanel(m_WindowNo, false, true);
			if (m_mTab.getTabNo() == 0)	//	initialize other tabs later
				m_tree.initTree(AD_Tree_ID);
			m_tree.addPropertyChangeListener(VTreePanel.NODE_SELECTION, this);
			graphPanel.add(m_tree, BorderLayout.CENTER);
			splitPane.setDividerLocation(250);
		//	splitPane.resetToPreferredSizes();
		}
		else    //  No Graphics - hide
		{
			graphPanel.setPreferredSize(new Dimension(0,0));
			splitPane.setDividerSize(0);
			splitPane.setDividerLocation(0);
		}

		//  Receive DataStatusChanged info from MTab
		m_mTab.addDataStatusListener(this);
		//  Receive vetoableChange info from MTable when saving
		m_mTab.getTableModel().addVetoableChangeListener(this);
		//	Selection Listener -> valueChanged
		vTable.getSelectionModel().addListSelectionListener(this);
		//  Navigation (RowChanged)
		m_mTab.addPropertyChangeListener(vTable);

		//  Update UI
		vTable.autoSize(true);

		setTabLevel(m_mTab.getTabLevel());

		//  Set initial presentation
		if (onlyMultiRow || !m_mTab.isSingleRow())
			switchMultiRow();
		else
			switchSingleRow();
	//	log.config( "GridController.dynInit (" + mTab.toString() + ") - fini");
		return true;
	}   //  initGrid

	/**************************************************************************
	 *  Init Grid.
	 *  <pre>
	 *  - Map table to model
	 *  - Update (multi-row) table info with renderers/editors
	 *  - build single-row panel
	 *  - initialize display
	 *  </pre>
	 *  @param mTab tab
	 *  @param onlyMultiRow only table
	 *  @param WindowNo window no
	 *  @param aPanel optional Application Panel for adding button listeners
	 * 	@param mWindow parent Window Model
	 *  @return true if initialized
	 */
	public boolean appendGridWithFormat (GridTab mTab, M_TabFormat m_TabFormat, ACPanel aPanel, MWindow mWindow )
	{
		log.config( "(" + mTab.toString() + ")");
		m_includedTabs.add( mTab );

		//  Single Row  -------------------------------------------------------
		if (!m_onlyMultiRow)
		{
			//	Set Softcoded Mnemonic &x
			int size = mTab.getFields().length;
			for (int i = 0; i < size; i++)
			{
				GridField mField = mTab.getField(i);
				if (mField.isDisplayed())
					vPanel.setMnemonic(mField);
			}   //  for all fields
			
			//	Add Fields
			for (int i = 0; i < size; i++)
			{
				GridField mField = mTab.getField(i);
				if (mField.isDisplayed())
				{
					VEditor vEditor = VEditorFactory.getEditor(mTab, mField, false);
					if (vEditor == null)
					{
						log.severe("Editor not created for " + mField.getColumnName());
						continue;
					}
					//  MField => VEditor - New Field value to be updated to editor
					mField.addPropertyChangeListener(vEditor);
					//  VEditor => this - New Editor value to be updated here (MTable)
					vEditor.addVetoableChangeListener(this);
					
					M_FieldFormat m_fieldFormat = M_FieldFormat.getM_FieldFormat(Env.getCtx(), mTab.getAD_Tab_ID(), mField.getAD_Column_ID() );
					//  Add to VPanel
					vPanel.addFieldWithFormat(vEditor, mField, m_fieldFormat);
					
					//  APanel Listen to buttons
					if (mField.getDisplayType() == DisplayType.Button && aPanel != null)
						((JButton)vEditor).addActionListener (aPanel);
				}
				
			}   //  for all fields
			
			if( m_TabFormat!=null ){
				vPanel.prepareComplete( m_TabFormat.getTab_ColCount() );
			} else {
				vPanel.prepareComplete( 0 );
			}
			
			//	No Included Grid Controller
			srPane.setResizeWeight(1);	//	top part gets all
			srPane.setDividerSize (0);
			srPane.setDividerLocation (9999);

			//  Use SR to size MR
			mrPane.setPreferredSize(vPanel.getPreferredSize());
		}   //  Single-Row

		//  Receive DataStatusChanged info from MTab
		mTab.addDataStatusListener(this);
		//  Receive vetoableChange info from MTable when saving
		mTab.getTableModel().addVetoableChangeListener(this);
		
		//  Set initial presentation
		if (m_onlyMultiRow || !mTab.isSingleRow())
			switchMultiRow();
		else
			switchSingleRow();
	//	log.config( "GridController.dynInit (" + mTab.toString() + ") - fini");
		return true;
	}   //  initGrid

	/**************************************************************************
	 *  Init Grid.
	 *  <pre>
	 *  - Map table to model
	 *  - Update (multi-row) table info with renderers/editors
	 *  - build single-row panel
	 *  - initialize display
	 *  </pre>
	 *  @param mTab tab
	 *  @param onlyMultiRow only table
	 *  @param WindowNo window no
	 *  @param aPanel optional Application Panel for adding button listeners
	 * 	@param mWindow parent Window Model
	 *  @return true if initialized
	 */
	public boolean appendGridController (ACGridController multiline_gc, GridTab mTab, M_TabFormat m_TabFormat, ACPanel aPanel, MWindow mWindow )
	{
		log.config( "(" + mTab.toString() + ")");
		m_includedTabs.add( mTab );

		//  Single Row  -------------------------------------------------------
		if (!m_onlyMultiRow)
		{
			//  Add to VPanel
			vPanel.addACGridController(multiline_gc, m_TabFormat);
			
			if( m_TabFormat!=null ){
				vPanel.prepareComplete( m_TabFormat.getTab_ColCount() );
			} else {
				vPanel.prepareComplete( 0 );
			}
			
			//	No Included Grid Controller
			srPane.setResizeWeight(1);	//	top part gets all
			srPane.setDividerSize (0);
			srPane.setDividerLocation (9999);

			//  Use SR to size MR
			mrPane.setPreferredSize(vPanel.getPreferredSize());
		}   //  Single-Row

		//  Receive DataStatusChanged info from MTab
		// mTab.addDataStatusListener(this);
		//  Receive vetoableChange info from MTable when saving
		// mTab.getTableModel().addVetoableChangeListener(this);
		
		switchSingleRow();
	//	log.config( "GridController.dynInit (" + mTab.toString() + ") - fini");
		return true;
	}   //  initGrid
	
	/**************************************************************************
	 *  Init Grid.
	 *  <pre>
	 *  - Map table to model
	 *  - Update (multi-row) table info with renderers/editors
	 *  - build single-row panel
	 *  - initialize display
	 *  </pre>
	 *  @param mTab tab
	 *  @param m_TabFormat tab format
	 *  @param WindowNo window no
	 *  @param aPanel optional Application Panel for adding button listeners
	 * 	@param mWindow parent Window Model
	 *  @return true if initialized
	 */
	public boolean initGridMultiLineTable (GridTab mTab, M_TabFormat m_TabFormat,  
		int WindowNo, ACPanel aPanel, MWindow mWindow )
	{
		log.config( "(" + mTab.toString() + ")");
		m_mTab = mTab;
		m_WindowNo = WindowNo;
		m_onlyMultiRow = true;
		setName("GC-" + mTab);

		//  Set up Multi Row Table
		vTable.setModel(m_mTab.getTableModel());

		//  Update Table Info -------------------------------------------------
		int size = setupVTableMultiLine (aPanel, m_mTab, m_TabFormat, vTable);
		
		//  Receive DataStatusChanged info from MTab
		m_mTab.addDataStatusListener(this);
		//  Receive vetoableChange info from MTable when saving
		m_mTab.getTableModel().addVetoableChangeListener(this);
		//	Selection Listener -> valueChanged
		vTable.getSelectionModel().addListSelectionListener(this);
		//  Navigation (RowChanged)
		m_mTab.addPropertyChangeListener(vTable);

		//  Update UI
		vTable.autoSize(true);

		setTabLevel(m_mTab.getTabLevel());

		switchMultiRow();
		
	//	log.config( "GridController.dynInit (" + mTab.toString() + ") - fini");
		return true;
	}   //  initGrid
	
	/**************************************************************************
	 *  Init Grid.
	 *  <pre>
	 *  - Map table to model
	 *  - Update (multi-row) table info with renderers/editors
	 *  - build single-row panel
	 *  - initialize display
	 *  </pre>
	 *  @param mTab tab
	 *  @param m_TabFormat tab Format
	 *  @param WindowNo window no
	 *  @param aPanel optional Application Panel for adding button listeners
	 * 	@param mWindow parent Window Model
	 *  @return true if initialized
	 */
	public boolean initGridMultiLineFormat (GridTab mTab, M_TabFormat m_TabFormat,  
		int WindowNo, ACPanel aPanel, MWindow mWindow )
	{
		log.config( "(" + mTab.toString() + ")");
		m_mTab = mTab;
		m_WindowNo = WindowNo;
		setName("GC-" + mTab);

		//  Set up Multi Row Table
		vTable.setModel(m_mTab.getTableModel());

		//  Update Table Info -------------------------------------------------
		int size = setupVTable (aPanel, m_mTab, vTable);
	
		if( m_TabFormat != null ) {
			vPanel.setRowAndCol( m_TabFormat.getTab_RowCount(), m_TabFormat.getTab_ColCount() );
		}
		
		//	Set Softcoded Mnemonic &x
		for (int i = 0; i < size; i++)
		{
			GridField mField = m_mTab.getField(i);
			if (mField.isDisplayed())
				vPanel.setMnemonic(mField);
		}   //  for all fields
		
		//	Add Fields
		for (int i = 0; i < size; i++)
		{
			GridField mField = m_mTab.getField(i);
			if (mField.isDisplayed())
			{
				VEditor vEditor = VEditorFactory.getEditor(m_mTab, mField, false);
				if (vEditor == null)
				{
					log.severe("Editor not created for " + mField.getColumnName());
					continue;
				}
				//  MField => VEditor - New Field value to be updated to editor
				mField.addPropertyChangeListener(vEditor);
				//  VEditor => this - New Editor value to be updated here (MTable)
				vEditor.addVetoableChangeListener(this);
				
				M_FieldFormat m_fieldFormat = M_FieldFormat.getM_FieldFormat(Env.getCtx(), m_mTab.getAD_Tab_ID(), mField.getAD_Column_ID() );
				//  Add to VPanel
				vPanel.addFieldWithFormat(vEditor, mField, m_fieldFormat);
				
				//  APanel Listen to buttons
				if (mField.getDisplayType() == DisplayType.Button && aPanel != null)
					((JButton)vEditor).addActionListener (aPanel);
			}
			
		}   //  for all fields
		
		if( m_TabFormat!=null ){
			vPanel.prepareComplete( m_TabFormat.getTab_ColCount() );
		} else {
			vPanel.prepareComplete( 0 );
		}
		
	
		//	No Included Grid Controller
		srPane.setResizeWeight(1);	//	top part gets all
		srPane.setDividerSize (0);
		srPane.setDividerLocation (9999);

		//  Use SR to size MR
		mrPane.setPreferredSize(vPanel.getPreferredSize());

		//  Receive DataStatusChanged info from MTab
		m_mTab.addDataStatusListener(this);
		//  Receive vetoableChange info from MTable when saving
		m_mTab.getTableModel().addVetoableChangeListener(this);
		//	Selection Listener -> valueChanged
		vTable.getSelectionModel().addListSelectionListener(this);
		//  Navigation (RowChanged)
		m_mTab.addPropertyChangeListener(vTable);

		//  Update UI
		vTable.autoSize(true);

		setTabLevel(m_mTab.getTabLevel());

		//  Set initial presentation
		switchSingleRow();
	//	log.config( "GridController.dynInit (" + mTab.toString() + ") - fini");
		return true;
	}   //  initGrid
	
	/**
	 *	Setup Multi-Row Table (add fields)
	 * 	@param aPanel Panel
	 * 	@param mTab Model Tab
	 * 	@param table JTable
	 * 	@return size
	 */
	private int setupVTableMultiLine (ACPanel aPanel, GridTab mTab, M_TabFormat m_TabFormat, VTable table)
	{
		if (!mTab.isDisplayed())
			return 0;
		
		int size = mTab.getFieldCount ();
		TableColumnModel tcm = table.getColumnModel();
		if (size != tcm.getColumnCount())
			throw new IllegalStateException("TableColumn Size <> TableModel");

		for (int i = 0; i < size; i++)
		{
			GridField mField = mTab.getField (i);
			
			TableColumn tc = tcm.getColumn(i);
			
			int displayLength = 8*mField.getDisplayLength();
			if( displayLength > 240 ){
				displayLength = 240;
			}
			tc.setMinWidth(displayLength);  
			//
			if (mField.getColumnName().equals(tc.getIdentifier().toString()))
			{
				if (mField.getDisplayType () == DisplayType.RowID)
				{
					tc.setCellRenderer (new VRowIDRenderer (false));
					tc.setCellEditor (new VRowIDEditor (false));
					tc.setHeaderValue ("");
					tc.setMaxWidth (2);
				}
				else
				{
					//  need to set CellEditor explicitly as default editor based on class causes problem (YesNo-> Boolean)
					if (mField.isDisplayed ())
					{
						tc.setCellRenderer (new VCellRenderer (mField));
						VCellEditor ce = new VCellEditor (mField);
						tc.setCellEditor (ce);
						//
						tc.setHeaderValue (mField.getHeader ());
						tc.setPreferredWidth (Math.max (mField.getDisplayLength (), 30));
						tc.setHeaderRenderer (new VHeaderRenderer (mField.getDisplayType ()));

						//  Enable Button actions in grid
						if (mField.getDisplayType () == DisplayType.Button)
						{
							VEditor button = ce.getEditor ();
							if (button != null && aPanel != null)
								((JButton)button).addActionListener (aPanel);
						}
					}
					else //  column not displayed
					{
						TableCellNone tcn = new TableCellNone(mField.getColumnName());
						tc.setCellRenderer (tcn);
						tc.setCellEditor (tcn);
						tc.setHeaderValue (null);
						tc.setMinWidth (0);
						tc.setMaxWidth (0);
						tc.setPreferredWidth (0);
					}
				}

			}	//	found field
			else
				log.log(Level.SEVERE, "TableColumn " + tc.getIdentifier ()
				  + " <> MField " + mField.getColumnName() + mField.getHeader());
		} 	//  for all fields
		
		return 1;
	}	//	setupVTable


	/**************************************************************************
	 *  Init Grid.
	 *  <pre>
	 *  - Map table to model
	 *  - Update (multi-row) table info with renderers/editors
	 *  - build single-row panel
	 *  - initialize display
	 *  </pre>
	 *  @param mTab tab
	 *  @param m_TabFormat tab format
	 *  @param WindowNo window no
	 *  @param aPanel optional Application Panel for adding button listeners
	 * 	@param mWindow parent Window Model
	 *  @return true if initialized
	 */
	public boolean initGridMultiLineOneColumnTable (GridTab mTab, M_TabFormat m_TabFormat,  
		int WindowNo, ACPanel aPanel, MWindow mWindow )
	{
		log.config( "(" + mTab.toString() + ")");
		m_mTab = mTab;
		m_WindowNo = WindowNo;
		m_onlyMultiRow = true;
		setName("GC-" + mTab);

		//  Set up Multi Row Table
		vTable.setModel(m_mTab.getTableModel());
		
		initToolBar( mTab, aPanel, mWindow );

		//  Update Table Info -------------------------------------------------
		int size = setupVTableMultiLineFormat (aPanel, m_mTab, m_TabFormat, vTable);
		
		//  Receive DataStatusChanged info from MTab
		m_mTab.addDataStatusListener(this);
		//  Receive vetoableChange info from MTable when saving
		m_mTab.getTableModel().addVetoableChangeListener(this);
		//	Selection Listener -> valueChanged
		vTable.getSelectionModel().addListSelectionListener(this);
		//  Navigation (RowChanged)
		m_mTab.addPropertyChangeListener(vTable);

		//  Update UI
//		vTable.autoSize(true);

		setTabLevel(m_mTab.getTabLevel());

		switchMultiRow();
		
	//	log.config( "GridController.dynInit (" + mTab.toString() + ") - fini");
		return true;
	}   //  initGrid
	
	/**
	 *	Setup Multi-Row Format (add fields)
	 * 	@param aPanel Panel
	 * 	@param mTab Model Tab
	 * 	@param table JTable
	 * 	@return size
	 */
	private int setupVTableMultiLineFormat (ACPanel aPanel, GridTab mTab, M_TabFormat m_TabFormat, VTable table)
	{
		if (!mTab.isDisplayed())
			return 0;
		
		String keyColumnName = mTab.getKeyColumnName();
		table.setOneColumnModel( true );
		TableColumnModel tcm = table.getColumnModel();
		for(int i=tcm.getColumnCount()-1; i>=0; i--){
			TableColumn tc = tcm.getColumn( i );
			if( !keyColumnName.equals(tc.getIdentifier().toString())){
				tcm.removeColumn(tc );
			}
		}
		TableColumn tc = tcm.getColumn( 0 );
		tc.setMinWidth( m_TabFormat.getTab_Width()-20 );
		
		VFormatedCellRenderer cr = new VFormatedCellRenderer (mTab, m_TabFormat); 
		tc.setCellRenderer ( cr );
		VFormatedCellEditor ce = new VFormatedCellEditor (mTab, m_TabFormat);
		tc.setCellEditor (ce);

		tc.setHeaderValue ( mTab.getName() );
		tc.setPreferredWidth (Math.max (m_TabFormat.getTab_Width()-20, 30));
		tc.setHeaderRenderer (new VHeaderRenderer (DisplayType.Button));

		int cellHeight = cr.getHeight();
		if( cellHeight < ce.getHeight() ){
			cellHeight = ce.getHeight();
		}
		if( cellHeight > 20 ){
			table.setRowHeight( cellHeight );
		}
		
		return 1;
	}	//	setupVTable

	/**************************************************************************
	 *  append multi Line One Column Table.
	 *  <pre>
	 *  - Map table to model
	 *  - Update (multi-row) table info with renderers/editors
	 *  - build single-row panel
	 *  - initialize display
	 *  </pre>
	 *  @param mTab tab
	 *  @param m_TabFormat tab format
	 *  @param WindowNo window no
	 *  @param aPanel optional Application Panel for adding button listeners
	 * 	@param mWindow parent Window Model
	 *  @return true if initialized
	 */
	public boolean appendMultiLineOneColumnTable (GridTab mTab, M_TabFormat m_TabFormat,  
		ACPanel aPanel, MWindow mWindow )
	{
		log.config( "(" + mTab.toString() + ")");
		m_includedTabs.add( mTab );

		//  Update Table Info -------------------------------------------------
		int size = appendVTableMultiLineFormat (aPanel, mTab, m_TabFormat, vTable);
		
		//  Receive DataStatusChanged info from MTab
		mTab.addDataStatusListener(this);
		//  Receive vetoableChange info from MTable when saving
		mTab.getTableModel().addVetoableChangeListener(this);
		//	Selection Listener -> valueChanged
		vTable.getSelectionModel().addListSelectionListener(this);
		//  Navigation (RowChanged)
		mTab.addPropertyChangeListener(vTable);

		switchMultiRow();
		
	//	log.config( "GridController.dynInit (" + mTab.toString() + ") - fini");
		return true;
	}   //  initGrid
	
	/**
	 *	Setup Multi-Row Format (add fields)
	 * 	@param aPanel Panel
	 * 	@param mTab Model Tab
	 * 	@param table JTable
	 * 	@return size
	 */
	private int appendVTableMultiLineFormat (ACPanel aPanel, GridTab mTab, M_TabFormat m_TabFormat, VTable table)
	{
		if (!mTab.isDisplayed())
			return 0;
		table.setOneColumnModel( true );
		
		TableColumnModel tcm = table.getColumnModel();
		TableColumn tc = tcm.getColumn( 0 );
		VFormatedCellRenderer cellRenderer = (VFormatedCellRenderer)tc.getCellRenderer(); 
		cellRenderer.appendFormatedCellRenderer( mTab );
		
		VFormatedCellEditor cellEditor = (VFormatedCellEditor)tc.getCellEditor();
		cellEditor.appendEditor( mTab );

		int cellHeight = cellRenderer.getHeight();
		if( cellHeight < cellEditor.getHeight() ){
			cellHeight = cellEditor.getHeight();
		}
		if( cellHeight > 20 ){
			table.setRowHeight( cellHeight );
		}
		
		return 1;
	}	//	setupVTable

	/**
	 * 	Get Title
	 *	@return title
	 */
	public String getTitle ()
	{
		return m_mTab.getName();
	}	//	getTitle
	
	/**
	 *	Setup Multi-Row Table (add fields)
	 * 	@param aPanel Panel
	 * 	@param mTab Model Tab
	 * 	@param table JTable
	 * 	@return size
	 */
	private int setupVTable (ACPanel aPanel, GridTab mTab, VTable table)
	{
		if (!mTab.isDisplayed())
			return 0;
		int size = mTab.getFieldCount ();
		TableColumnModel tcm = table.getColumnModel();
		if (size != tcm.getColumnCount())
			throw new IllegalStateException("TableColumn Size <> TableModel");

		for (int i = 0; i < size; i++)
		{
			GridField mField = mTab.getField (i);
			
			TableColumn tc = tcm.getColumn(i);
			
//			tc.setMinWidth(30);
//			Jirimuto Modified 20060401
			int displayLength = 8*mField.getDisplayLength();
			if( displayLength > 240 ){
				displayLength = 240;
			}
			tc.setMinWidth(displayLength);  
			//
			if (mField.getColumnName().equals(tc.getIdentifier().toString()))
			{
				if (mField.getDisplayType () == DisplayType.RowID)
				{
					tc.setCellRenderer (new VRowIDRenderer (false));
					tc.setCellEditor (new VRowIDEditor (false));
					tc.setHeaderValue ("");
					tc.setMaxWidth (2);
				}
				else
				{
					//  need to set CellEditor explicitly as default editor based on class causes problem (YesNo-> Boolean)
					if (mField.isDisplayed ())
					{
						tc.setCellRenderer (new VCellRenderer (mField));
						VCellEditor ce = new VCellEditor (mField);
						tc.setCellEditor (ce);
						//
						tc.setHeaderValue (mField.getHeader ());
						tc.setPreferredWidth (Math.max (mField.getDisplayLength (), 30));
						tc.setHeaderRenderer (new VHeaderRenderer (mField.getDisplayType ()));

						//  Enable Button actions in grid
						if (mField.getDisplayType () == DisplayType.Button)
						{
							VEditor button = ce.getEditor ();
							if (button != null && aPanel != null)
								((JButton)button).addActionListener (aPanel);
						}
					}
					else //  column not displayed
					{
						TableCellNone tcn = new TableCellNone(mField.getColumnName());
						tc.setCellRenderer (tcn);
						tc.setCellEditor (tcn);
						tc.setHeaderValue (null);
						tc.setMinWidth (0);
						tc.setMaxWidth (0);
						tc.setPreferredWidth (0);
					}
				}
			//	System.out.println ("TableColumnID " + tc.getIdentifier ()
			//	  + "  Renderer=" + tc.getCellRenderer ()
			//	  + mField.getHeader ());

			}	//	found field
			else
				log.log(Level.SEVERE, "TableColumn " + tc.getIdentifier ()
				  + " <> MField " + mField.getColumnName() + mField.getHeader());
		} 	//  for all fields
		return size;
	}	//	setupVTable

	/**
	 * 	Activate Grid Controller.
	 * 	Called by APanel when GridController is displayed (foreground)
	 */
	public void activate ()
	{
		//	Tree to be initiated on second/.. tab
		if (m_mTab.isTreeTab() && m_mTab.getTabNo() != 0)
		{
			int AD_Tree_ID = Env.getContextAsInt (Env.getCtx(), m_WindowNo, "AD_Tree_ID");
			if (AD_Tree_ID == 0)
				AD_Tree_ID = MTree.getDefaultAD_Tree_ID (
					Env.getAD_Client_ID(Env.getCtx()), m_mTab.getKeyColumnName());
			m_tree.initTree (AD_Tree_ID);
		}
	}	//	activate


	/**
	 *  Register ESC Actions
	 *  - overwrite VTable's Keystroks assigment for ESC
	 *  @param aIgnore ignore
	 */
	public void registerESCAction (AppsAction aIgnore)
	{
		// Jirimuto added for One Column table.
		if( vTable.isOneColumnModel() ) {
			int c = VTable.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT;
			vTable.getInputMap(c).put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), aIgnore.getName());
			vTable.getActionMap().put(aIgnore.getName(), aIgnore);
		}
	//	AEnv.printActionInputMap(vTable);
	}   //  registerESCAction

	/**
	 *  Query Tab and resize Table
	 *  (called from APanel)
	 *  @param onlyCurrentRows only current rows
	 *  @param onlyCurrentDays how many days back
	 */
	public void query (boolean onlyCurrentRows, int onlyCurrentDays)
	{
		//  start loading while building screen
		m_mTab.query(onlyCurrentRows, onlyCurrentDays);
		//  Update UI
		if (!isSingleRow())
			vTable.autoSize(true);
	}   //  query

	
	/**************************************************************************
	 *  Switch from single to multi & vice versa
	 */
	public void switchRowPresentation()
	{
		stopEditor(true);
		if (m_singleRow)
			switchMultiRow();
		else
			switchSingleRow();
	}   //  switchRowPresentation

	/**
	 *  Switch to SingleRow Presentation
	 */
	public void switchSingleRow()
	{
		if (m_onlyMultiRow)
			return;
		cardLayout.first(cardPanel);
		m_singleRow = true;
		dynamicDisplay(m_mTab, 0);
	//	vPanel.requestFocus();
	}   //  switchSingleRow

	/**
	 *  Switch to MultiRow Presentation
	 */
	public void switchMultiRow()
	{
		cardLayout.last(cardPanel);
		m_singleRow = false;
		vTable.autoSize(true);	//	resizes
	//	vTable.requestFocus();
	}   //  switchSingleRow

	/**
	 *  Is Single Row presentation
	 *  @return true if Single Row is displayed
	 */
	public boolean isSingleRow()
	{
		return m_singleRow;
	}   //  isSingleRow

	
	/**************************************************************************
	 *  Remove Listener - pass on to MTab
	 *  @param l listener
	 */
	public synchronized void removeDataStatusListener(DataStatusListener l)
	{
		m_mTab.removeDataStatusListener(l);
	}   //  removeDataStatusListener

	/**
	 *  Add Data Status Listener - pass on to MTab
	 *  @param l listener
	 */
	public synchronized void addDataStatusListener(DataStatusListener l)
	{
		m_mTab.addDataStatusListener(l);
	}

	/**
	 *  Data Status Listener - for MTab events.
	 *  <p>
	 *  Callouts are processed here for GUI changes
	 *  - same as in MTab.setValue for batch changes
	 *  <p>
	 *  calls dynamicDisplay
	 *  @param e event
	 */
	public void dataStatusChanged(DataStatusEvent e)
	{
		if( dataStatusChangeAcvtive )
			return;
		// if (e.getChangedColumn() == 0)
		//	return;
		dataStatusChangeAcvtive = true;
		boolean changedOnChildTable = false;
		int col = e.getChangedColumn();
		GridTab mTab = m_mTab;
		if( e.getSource() instanceof GridTable ){
			GridTable tmp_mTable = (GridTable)e.getSource();
			if( tmp_mTable != mTab.getTableModel() ){
				for( int i=0; i<m_includedTabs.size(); i++ ){
					GridTab tmp_mTab = m_includedTabs.get(i);
					if( tmp_mTable == tmp_mTab.getTableModel() ){
						mTab = tmp_mTab;
						changedOnChildTable = true;
						break;
					}
				}
			}
		}
		
		log.config("(" + mTab + ") Col=" + col + ": " + e.toString());

		//  Process Callout only for specific columns
		if (col != 0 && !changedOnChildTable )
		{
			GridField mField = mTab.getField(col);
			if (mField != null && mField.getCallout().length() > 0)
			{
				String msg = mTab.processFieldChange(mField);     //  Dependencies & Callout
				if (msg.length() > 0)
					ADialog.error(m_WindowNo, this, msg);
			}
		}
		dynamicDisplay(mTab, col);
		
		// Jirimuto added for One Column table.
		int rowTable = vTable.getSelectedRow();
		int rowCurrent = m_mTab.getCurrentRow();
		if( rowTable != rowCurrent ){
			if( rowTable == -1 ){
				vTable.setRowSelectionInterval(rowCurrent, rowCurrent); //  causes this method to be called again
				rowTable = rowCurrent;
			}
		}
		// Jirimuto added for One Column table.
		if( vTable.isOneColumnModel() && !vTable.isEditing() ) {
			GridTab parentTab = m_mTab.getParentTab();
			if( parentTab==null || !parentTab.isDeleteing() ){
				vTable.editCellAt( rowCurrent, 0 ); // Dangerous on delete
			}
		}
		
		dataStatusChangeAcvtive = false;

		toolbarStatusAdjust();
		
	}   //  dataStatusChanged

	
	/**************************************************************************
	 *  List Selection Listener (VTable) - row changed
	 *  @param e event
	 */
	public void valueChanged(ListSelectionEvent e)
	{
		//	nothing or initiated by mouse (wait for "real" one)
		if (e.getValueIsAdjusting())
			return;
		//  no rows
		if (m_mTab.getRowCount() == 0)
			return;

	//	vTable.stopEditor(graphPanel);
		int rowTable = vTable.getSelectedRow();
		int rowCurrent = m_mTab.getCurrentRow();
		log.config("(" + m_mTab.toString() + ") Row in Table=" + rowTable + ", in Model=" + rowCurrent);

		if (rowTable == -1)  //  nothing selected
		{
			if (rowCurrent >= 0)
			{
				vTable.setRowSelectionInterval(rowCurrent, rowCurrent); //  causes this method to be called again
				// Jirimuto added for One Column table.
				if( vTable.isOneColumnModel() ) {
					GridTab parentTab = m_mTab.getParentTab();
					if( parentTab==null || !parentTab.isDeleteing() ){
						vTable.editCellAt( rowCurrent, 0 ); // Dangerous on delete
					}
				}
				toolbarStatusAdjust();
				
				return;
			}
		}
		else
		{
			if (rowTable != rowCurrent)
				m_mTab.navigate(rowTable);
			dynamicDisplay(m_mTab, 0);
		}
		
		// Jirimuto added for One Column table.
		if( vTable.isOneColumnModel() ) {
			if( rowTable!=-1 && rowTable != rowCurrent ){
				vTable.editCellAt( rowTable, 0 ); // Dangerous
			}
		}

		//	TreeNavigation - Synchronize 	-- select node in tree
		if (m_tree != null)
			m_tree.setSelectedNode (m_mTab.getRecord_ID());	//	ignores new (-1)

	//	log.config( "GridController.valueChanged (" + m_mTab.toString() + ") - fini",
	//		"Row in Table=" + rowTable + ", in Model=" + rowCurrent);

		//	Query Included Tab
		if (vIncludedGC != null)
			vIncludedGC.getMTab().query(false, 0);
		
		toolbarStatusAdjust();
		
	}   //  valueChanged

	/**
	 *  PropertyChange Listener - Tree Panel - node selection
	 *  @param e event
	 */
	public void propertyChange(PropertyChangeEvent e)
	{
		// System.out.println("propertyChange");
		// System.out.println(e);
		if (e == null)
			return;
		Object value = e.getNewValue();
		if (value == null)
			return;
		log.config(e.getPropertyName() + "=" + value
			+ " - " + value.getClass().toString());
		if (!(value instanceof MTreeNode))
			return;

		//  We Have a TreeNode
		int nodeID = ((MTreeNode)value).getNode_ID();
		//  root of tree selected - ignore
		if (nodeID == 0)
			return;

		//  Search all rows for mode id
		int size = m_mTab.getRowCount();
		int row = -1;
		for (int i = 0; i < size; i++)
		{
			if (m_mTab.getKeyID(i) == nodeID)
			{
				row = i;
				break;
			}
		}
		if (row == -1)
		{
			log.log(Level.SEVERE, "Tab does not have ID with Node_ID=" + nodeID);
			return;
		}

		//  Navigate to node row
		m_mTab.navigate(row);
		
	}   //  propertyChange

	/**
	 *  Dynamic Display.
	 *  - Single Row Screen layout and update of dynamic Lookups
	 *  <p>
	 *  Single Row layout:
	 *  the components's name is the ColumnName; if it matches, the
	 *  MField.isDisplayed(true) is used to determine if it is visible
	 *  if the component is a VEditor, setEnabled is set from the MField
	 *  <p>
	 *  Multi Row layout is not changed:
	 *  VCellRenderer calls JTable.isCellEditable -> checks MField.isEditable (Active, isDisplayed)
	 *  VCellEditor.isCellEditable calls MField.isEditable(true) <br>
	 *  If a column is not displayed, the width is set to 0 in dynInit
	 *  <p>
	 *  Dynamic update of data is handeled in VLookup.focusGained/Lost.
	 *  When focus is gained the model is temporarily updated with the
	 *  specific validated data, if lost, it is switched back to the
	 *  unvalidated data (i.e. everything). This allows that the display
	 *  methods have a lookup to display. <br>
	 *  Here: if the changed field has dependents and the dependent
	 *  is a Lookup and this lookup has a dynamic dependence of the changed field,
	 *  the value of that field is set to null (in MTab.processDependencies -
	 *  otherwise it would show an invalid value).
	 *  As Editors listen for value changed of their MField, the display is updated.
	 *  <p>
	 *  Called from GridController.valueChanged/dataStatusChanged, APane;.stateChanged/unlock/cmd_...
	 *  @param col selective column number or 0 if all
	 */
	public void dynamicDisplay (GridTab mTab, int col)
	{
		//  complete single row re-display
		boolean noData = mTab.getRowCount() == 0;
		log.config(mTab.toString() + " - Rows=" + mTab.getRowCount());
		if( vTable.isOneColumnModel() && vTable.getCellEditor()!=null && vTable.getCellEditor() instanceof VFormatedCellEditor){
			
			Component[] vVertComp = ((VACPanel)((VFormatedCellEditor)vTable.getCellEditor()).getEditor()).getComponents();
			for( int k=0; k<vVertComp.length; k++) {
				Component[] vHorzComp = ((JComponent)vVertComp[k]).getComponents();
				for( int j=0; j<vHorzComp.length; j++) {
					Component[] comp = ((JComponent)vHorzComp[j]).getComponents();
					for (int i = 0; i < comp.length; i++)
					{
						String columnName = comp[i].getName();
						if (columnName != null)
						{
							GridField mField = mTab.getField(columnName);
							if (mField != null)
							{
								if (mField.isDisplayed(true))       //  check context
								{
									if (!comp[i].isVisible())
										comp[i].setVisible(true);                   //  visibility
									if (comp[i] instanceof VEditor)
									{
										VEditor ve = (VEditor)comp[i];
										if (noData)
											ve.setReadWrite(false);
										else
										{
											boolean rw = mField.isEditable(true);       //  r/w - check Context
											ve.setReadWrite(rw);
										//	Log.trace(7, "GridController.dynamicDisplay RW=" + rw, mField);
											boolean manMissing = false;
										//  least expensive operations first        //  missing mandatory
											if (rw && mField.getValue() == null && mField.isMandatory(true))    //  check context
												manMissing = true;
											ve.setBackground(manMissing || mField.isError());
										}
									}
								}
								else if (comp[i].isVisible())
									comp[i].setVisible(false);
							}
						}
					}
				}
			}   //  all components
		}
		//	log.config( "GridController.dynamicDisplay (" + m_mTab.toString() + ") SingleRow=" + isSingleRow() + ", OnlyMultiRow=" + m_onlyMultiRow);
		//	Don't update if multi-row		
		if (!isSingleRow() || m_onlyMultiRow )
			return;
		if (!mTab.isOpen())
			return;
		//  Selective
		if (col != 0)
		{
			GridField changedField = mTab.getField(col);
			String columnName = changedField.getColumnName();
			ArrayList dependants = mTab.getDependantList(columnName);
			log.config("(" + mTab.toString() + ") "
				+ columnName + " - Dependents=" + dependants.size());
			//	No Dependents and no Callout - Set just Background
			if (dependants.size() == 0 && changedField.getCallout().length() > 0)
			{
				Component[] comp = vPanel.getComponents();
				for (int i = 0; i < comp.length; i++)
				{
					if (columnName.equals(comp[i].getName ()) && comp[i] instanceof VEditor)
					{
						VEditor ve = (VEditor)comp[i];
						boolean manMissing = false;
						boolean noValue = changedField.getValue() == null || changedField.getValue().toString().length() == 0;
						if (noValue && changedField.isEditable(true) && changedField.isMandatory(true))    //  check context
							manMissing = true;
						ve.setBackground(manMissing || changedField.isError());
						break;
					}
				}
				return;
			}
		}   //  selective

		//  All Components in vPanel (Single Row)
		Component[] vVertComp = vPanel.getComponents();
		for( int k=0; k<vVertComp.length; k++) {
			Component[] vHorzComp = ((JComponent)vVertComp[k]).getComponents();
			for( int j=0; j<vHorzComp.length; j++) {
				Component[] comp = ((JComponent)vHorzComp[j]).getComponents();
				for (int i = 0; i < comp.length; i++)
				{
					String columnName = comp[i].getName();
					if (columnName != null)
					{
						GridField mField = mTab.getField(columnName);
						if (mField != null)
						{
							if (mField.isDisplayed(true))       //  check context
							{
								if (!comp[i].isVisible())
									comp[i].setVisible(true);                   //  visibility
								if (comp[i] instanceof VEditor)
								{
									VEditor ve = (VEditor)comp[i];
									if (noData)
										ve.setReadWrite(false);
									else
									{
										boolean rw = mField.isEditable(true);       //  r/w - check Context
										ve.setReadWrite(rw);
									//	Log.trace(7, "GridController.dynamicDisplay RW=" + rw, mField);
										boolean manMissing = false;
									//  least expensive operations first        //  missing mandatory
										if (rw && mField.getValue() == null && mField.isMandatory(true))    //  check context
											manMissing = true;
										ve.setBackground(manMissing || mField.isError());
									}
								}
							}
							else if (comp[i].isVisible())
								comp[i].setVisible(false);
						}
					}
				}
			}
		}   //  all components
		
		log.config(mTab.toString() + " - fini - " + (col==0 ? "complete" : "seletive"));
	}   //  dynamicDisplay

	/**
	 *  Row Changed - synchronize with Tree
	 *
	 *  @param  save    true the row was saved (changed/added), false if the row was deleted
	 *  @param  keyID   the ID of the row changed
	 */
	public void rowChanged (boolean save, int keyID)
	{
		if (m_tree == null || keyID <= 0)
			return;
		String name = (String)m_mTab.getValue("Name");
		String description = (String)m_mTab.getValue("Description");
		Boolean IsSummary = (Boolean)m_mTab.getValue("IsSummary");
		String imageIndicator = (String)m_mTab.getValue("Action");  //  Menu - Action
		//
		m_tree.nodeChanged(save, keyID, name, description, 
			IsSummary.booleanValue(), imageIndicator);
	}   //  rowChanged
	
	/**************************************************************************
	 *  Vetoable Change Listener.
	 * 	Called from VEditor
	 *  <pre>
	 *  - for Save Confirmation dialog
	 *  - for Single Row from VEditor: Update MTable
	 *  </pre>
	 *  @param e event
	 *  @throws PropertyVetoException
	 */
	public void vetoableChange(PropertyChangeEvent e) throws PropertyVetoException
	{
		//  Get Row/Col Info
		GridTab   mTab = m_mTab;
		GridTable mTable = m_mTab.getTableModel();
		int row = mTab.getCurrentRow();
		int col = mTable.findColumn(e.getPropertyName());
		
		// Jirimuto modified 
		if( col < 0 ){
			for(int i=0; i<m_includedTabs.size(); i++ ){
				GridTab includedTab = (GridTab)m_includedTabs.get( i );
				GridTable includedTable = includedTab.getTableModel();
				col = includedTable.findColumn(e.getPropertyName());
				if( col >= 0 ){
					mTab = includedTab;
					row = mTab.getCurrentRow();
					mTable = includedTable;
					break;
				}
			}
			
		}
		
		if (mTab.isProcessed())		//	only active records
		{
			Object source = e.getSource();
			if (source instanceof VEditor)
			{
				if (!((VEditor)source).isReadWrite())
				{
					log.config("(" + mTab.toString() + ") " + e.getPropertyName());
					return;
				}
			}
			else
			{
				log.config("(" + mTab.toString() + ") " + e.getPropertyName());
				return;
			}
		}
		log.config("(" + mTab.toString() + ") "
			+ e.getPropertyName() + "=" + e.getNewValue() + " (" + e.getOldValue() + ") "
			+ (e.getOldValue() == null ? "" : e.getOldValue().getClass().getName()));
		

		//  Save Confirmation dialog    MTable-RowSave
		if (e.getPropertyName().equals(GridTable.PROPERTY))
		{
			//  throw new PropertyVetoException calls this method (??) again
			if (m_vetoActive)
			{
				m_vetoActive = false;
				return;
			}
			if (!Env.isAutoCommit(Env.getCtx(), m_WindowNo) || mTab.getCommitWarning().length() > 0)
			{
				if (!ADialog.ask(m_WindowNo, this, "SaveChanges?", mTab.getCommitWarning()))
				{
					m_vetoActive = true;
					throw new PropertyVetoException ("UserDeniedSave", e);
				}
			}
			return;
		}   //  saveConfirmation

		//
		if (e.getNewValue() == null && e.getOldValue() != null 
			&& e.getOldValue().toString().length() > 0)		//	some editors return "" instead of null
			mTable.setChanged (true);
		else
		{
		//	mTable.setValueAt (e.getNewValue(), row, col, true);
			mTable.setValueAt (e.getNewValue(), row, col);	//	-> dataStatusChanged -> dynamicDisplay
			//	Force Callout
			if (e.getPropertyName().equals("S_ResourceAssignment_ID"))
			{
				GridField mField = mTab.getField(col);
				if (mField != null && mField.getCallout().length() > 0)
					mTab.processFieldChange(mField);     //  Dependencies & Callout
			}
		}

		if( vTable.isOneColumnModel() ) {
			toolbarStatusAdjust();
		}
		
	//	log.config( "GridController.vetoableChange (" + m_mTab.toString() + ") - fini", e.getPropertyName() + "=" + e.getNewValue());
	}   //  vetoableChange

	
	/**************************************************************************
	 *  Get Model Tab
	 *  @return Model Tab
	 */
	public GridTab getMTab()
	{
		return m_mTab;
	}   //  getMTab

	/**
	 *  Get VTable
	 *  @return VTable
	 */
	public VTable getTable()
	{
		return vTable;
	}   //  getTable

	
	/**
	 * 	Set Window level Mnemonics
	 *	@param set true if set otherwise unregiser
	 */
	public void setMnemonics (boolean set)
	{
		if (vPanel != null)
			vPanel.setMnemonics(set);
	}	//	setMnemonics
	
	/**
	 *  Stop Table & SR Editors and move focus to graphPanel
	 *  @param saveValue save value
	 */
	public void stopEditor (boolean saveValue)
	{
		log.config("(" + m_mTab.toString() + ") TableEditing=" + vTable.isEditing());

		//  MultiRow - remove editors
		vTable.stopEditor(saveValue);

		//  SingleRow - stop editors by changing focus
		if (m_singleRow)
			vPanel.transferFocus();
		//	graphPanel.requestFocus();
		//
	//	log.config( "GridController.stopEditor (" + m_mTab.toString() + ") - fini",
	//		"Editing=" + vTable.isEditing());
	}   //  stopEditors


	public void mouseClicked(MouseEvent e)
	{
		log.finest("" + this + " - " + e);
	}
	public void mousePressed(MouseEvent e)
	{
		log.finest("" + this + " - " + e);
		if( vTable.isOneColumnModel() ) {
			toolbarStatusAdjust();
		}
	}
	public void mouseReleased(MouseEvent e)
	{
		log.finest("" + this + " - " + e);
		if( vTable.isOneColumnModel() ) {
			toolbarStatusAdjust();
		}
	}
	public void mouseEntered(MouseEvent e)
	{
		log.finest("" + this + " - " + e);
	}
	public void mouseExited(MouseEvent e)
	{
		log.finest("" + this + " - " + e);
	}

	/**
	 *  Action Performed (Action Listener)
	 *  @param e ActionEvent
	 */
	public void actionPerformed(ActionEvent e)
	{
	//	log.info(e.getActionCommand());
		Object source = e.getSource();
		if (source == bIgnore){
			cmd_lineIgnore();
		} else if (source == bNew){
			cmd_lineNew( false );
		} else if (source == bCopy) {
			cmd_lineNew( true );
		} else if (source == bDelete) {
			cmd_lineDelete( );
		} else if (source == bSave){
			cmd_lineSave( false );
		} else if (source == bRefresh){
			cmd_lineRefresh( );
		}
	}   //  actionPerformed

	/**
	 *  Create New Record
	 *  @param copy true if current record is to be copied
	 */
	private void cmd_lineNew (boolean copy)
	{
		log.config("copy=" + copy);
		if (!m_mTab.isInsertRecord())
		{
			log.warning("Insert Record disabled for Tab");
			return;
		}
		this.stopEditor(true);
		cmd_lineSave(false);
		m_mTab.dataNew (copy);
		
//		Jirimuto added for One Column table.
		int currentRow = m_mTab.getCurrentRow();
		if( vTable.isOneColumnModel() ) {
			vTable.removeEditor();
			vTable.editCellAt( currentRow, 0 );
			// m_mTab.getTableModel().setChanged( true );
		}
		this.dynamicDisplay(m_mTab, 0);
		m_mTab.getTableModel().setChanged( true );
		if( vTable.isOneColumnModel() ) {
			toolbarStatusAdjust();
		}
	}   //  cmd_new

	/**
	 *  Confirm & delete record
	 */
	private void cmd_lineDelete()
	{
		this.stopEditor(true);
		if (m_mTab.isReadOnly())
			return;
		int keyID = m_mTab.getRecord_ID();
		if (ADialog.ask(m_WindowNo, this, "DeleteRecord?"))
			if (m_mTab.dataDelete())
				this.rowChanged(false, keyID);
		this.dynamicDisplay(m_mTab, 0);
	}   //  cmd_delete

	/**
	 *  If required ask if you want to save and save it
	 *  @param manualCmd true if invoked manually (i.e. force)
	 *  @return true if saved
	 */
	private boolean cmd_lineSave (boolean manualCmd)
	{
		log.config("Manual=" + manualCmd);
		boolean saveOK = true;

		//  manually initiated
		boolean retValue = m_mTab.dataSave(manualCmd);
		
		saveMultiLineTable();
		
		this.rowChanged(true, m_mTab.getRecord_ID());
		if (manualCmd)
			this.dynamicDisplay(m_mTab, 0);
		
		return retValue;
	}   //  cmd_lineSave

	/**
	 *  Line Ignore
	 */
	private void cmd_lineIgnore()
	{
		this.stopEditor(true);
		m_mTab.dataIgnore();
		m_mTab.dataRefreshAll();
		this.dynamicDisplay(m_mTab, 0);
	}   //  cmd_lineignore

	/**
	 *  Line Refresh
	 */
	private void cmd_lineRefresh()
	{
		cmd_lineSave(false);
		m_mTab.dataRefreshAll();
		this.dynamicDisplay(m_mTab, 0);
	}   //  cmd_refresh

	public void saveMultiLineTable(){
		
		if( vTable.isOneColumnModel() ) {
			TableCellEditor cellEditor = vTable.getCellEditor();
			if( cellEditor instanceof VFormatedCellEditor ){
				((VFormatedCellEditor)cellEditor).setCellEditorValue( m_mTab.getRecord_ID());
			}
			toolbarStatusAdjust();
		}
	}
	
	/**
	 *  RefreshAll added by Jirimuto
	 */
	public void freshAll()
	{
		m_mTab.dataRefreshAll();
		int currentRow = m_mTab.getCurrentRow();
		if( vTable.isOneColumnModel() ) {
			vTable.editCellAt( currentRow, 0 );
		}
		dynamicDisplay(m_mTab, 0);
		if( vTable.isOneColumnModel() ) {
			toolbarStatusAdjust();
		}
	}   //  cmd_refresh

	
	private void toolbarStatusAdjust(){
		
		if( m_mTab.getParentTab() != null ) {
			GridTab m_parentTab = m_mTab.getParentTab();
			if( m_parentTab.isProcessed() ){
				bIgnore.setEnabled( false );
				bNew.setEnabled( false );
				bSave.setEnabled( false );
				bCopy.setEnabled( false );
				bDelete.setEnabled( false );
				bRefresh.setEnabled( true );
				return;
			}
		}
		boolean changed = m_mTab.needSave(true, true) || ( (m_mTab.getRowCount()>0) && m_mTab.getCurrentRow()>=0 && m_mTab.getRecord_ID()==-1);
		boolean isEditing = vTable.isEditing() & (m_mTab.getRowCount()>0) & (m_mTab.getCurrentRow()>=0)
							& m_mTab.getRecord_ID() >= 0 ;
		if( changed ){
			bIgnore.setEnabled( true );
			bNew.setEnabled( false );
			bSave.setEnabled( true );
			bRefresh.setEnabled( false );
		} else {
			bIgnore.setEnabled( false );
			bNew.setEnabled( true );
			bSave.setEnabled( false );
			bRefresh.setEnabled( true );
		}
		
		if( isEditing ){
			bCopy.setEnabled( true );
			bDelete.setEnabled( true );
		} else {
			bCopy.setEnabled( false );
			bDelete.setEnabled( false );
		}
		
		if( m_mTab.getParentTab() != null && m_mTab.getParentTab().getRecord_ID() < 0 ){			
			bIgnore.setEnabled( false );
			bNew.setEnabled( false );
			bSave.setEnabled( false );
			bCopy.setEnabled( false );
			bDelete.setEnabled( false );
			bRefresh.setEnabled( true );
		}
	}

}   //  GridController
