/******************************************************************************
 * The contents of this file are subject to the   Compiere License  Version 1.1
 * ("License"); You may not use this file except in compliance with the License
 * You may obtain a copy of the License at http://www.compiere.org/license.html
 * Software distributed under the License is distributed on an  "AS IS"  basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 * The Original Code is Compiere ERP & CRM Smart Business Solution. The Initial
 * Developer of the Original Code is Jorg Janke. Portions created by Jorg Janke
 * are Copyright (C) 1999-2005 Jorg Janke.
 * All parts are Copyright (C) 1999-2005 ComPiere, Inc.  All Rights Reserved.
 * Contributor(s): ______________________________________.
 *****************************************************************************/
package org.compiere.grid;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.InputMap;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.table.TableModel;

import org.compiere.grid.ed.VCellEditor;
import org.compiere.grid.ed.VEditor;
import org.compiere.grid.ed.VFormatedCellEditor;
import org.compiere.model.GridTab;
import org.compiere.model.GridTable;
import org.compiere.swing.CTable;
import org.compiere.util.CLogger;

/**
 *  Table Grid based on CTable.
 * 	Used in GridController
 *
 * 	@author 	Jorg Janke
 * 	@version 	$Id: VTable.java,v 1.1 2006/09/25 14:19:51 hans Exp $
 */
public final class VTable extends CTable 
	implements PropertyChangeListener
{
	/**
	 *	Default Constructor
	 */
	public VTable()
	{
		super();
		setAutoscrolls(true);
		
		/**
		 *  Added By Jirimuto 2006/04/02
		 *  The following code is added for Grid Control for 
		 *  [Enter] key act as [Tab] key
		 */
		
		InputMap im
	      = super.getInputMap(JTable.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);

	    // Register [Tab] key Action Name to [Enter] key
	    KeyStroke tab = KeyStroke.getKeyStroke(KeyEvent.VK_TAB, 0);
	    KeyStroke enter = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0);
	    im.put(enter, im.get(tab)); // im.get() is getting existing name of the tab key stroke
		
	    /* 
	     * New action definition for [Tab] key.
	     * This will be used by [Enter] key as well.
	     */
	    final Action oldTabAction = super.getActionMap().get(im.get(tab));
	    Action tabAction = new AbstractAction(){
	      public void actionPerformed(ActionEvent e){

	        CTable table = (CTable)e.getSource();
	        int rowCount = table.getRowCount();
	        int columnCount = table.getColumnCount();
	        int row = table.getSelectedRow();
	        int column = table.getSelectedColumn();
	        if( !table.isOneColumnModel() ) {
		        oldTabAction.actionPerformed(e);
		        // move to next editable cell 
		        while (! table.isCellEditable(row, column) ){
		          column += 1;
		          if (column == columnCount){
		            column = 0;
		            row +=1;
		          }
		          if (row == rowCount){
		            row = 0;
		          }
		          if (row == table.getSelectedRow()
		              &&  column == table.getSelectedColumn()){
		            break; 
		          }
		        }
		        table.changeSelection(row, column, false, false);
	        } else {
	        	;
	        	
	        }
	        
	      }
	    };
	    
	    super.getActionMap().put(im.get(tab), tabAction);
	    
	}	//	VTable

	/**	Logger			*/
	private static CLogger log = CLogger.getCLogger(VTable.class);
	
	private char currentKeyChar = 0;
	
	/**
	 *  Property Change Listener for CurrentRow.
	 *  - Selects the current row if not already selected
	 *  - Required when navigating via Buttons
	 *  @param evt event
	 */
	public void propertyChange(PropertyChangeEvent evt)
	{
		// Jirimuto added for child tab property change 
		if( !getModel().equals( evt.getSource() ))
			return;
		
	//	log.config(evt);
		if (evt.getPropertyName().equals(GridTab.PROPERTY))
		{
			int row = ((Integer)evt.getNewValue()).intValue();
			int selRow = getSelectedRow();
			if (row == selRow)
				return;
			log.config(GridTab.PROPERTY + "=" + row + " from " + selRow);

			setRowSelectionInterval(row,row);
			if( super.isOneColumnModel() ) {
				
			
			}
		    Rectangle cellRect = getCellRect(row, 1, false);
		    if (cellRect != null)
		    	scrollRectToVisible(cellRect);
			log.config(GridTab.PROPERTY + "=" + row + " from " + selRow);
		}
	}   //  propertyChange

	/**
	 *	Get ColorCode for Row.
	 *  <pre>
	 *	If numerical value in compare column is
	 *		negative = -1,
	 *      positive = 1,
	 *      otherwise = 0
	 *  </pre>
	 *  @param row row
	 *  @return color code
	 */
	public int getColorCode (int row)
	{
		return ((GridTable)getModel()).getColorCode(row);
	}   //  getColorCode

	/**
	 *  Sort Table
	 *  @param modelColumnIndex model column sort index
	 */
	protected void sort (int modelColumnIndex)
	{
		int rows = getRowCount();
		if (rows == 0)
			return;
		//
		TableModel model = getModel();
		if (!(model instanceof GridTable))
		{
			super.sort(modelColumnIndex);
			return;
		}

		//  other sort column
		if (modelColumnIndex != p_lastSortIndex)
			p_asc = true;
		else
			p_asc = !p_asc;

		p_lastSortIndex = modelColumnIndex;
		//
		log.config("#" + modelColumnIndex
			+ " - rows=" + rows + ", asc=" + p_asc);

		((GridTable)model).sort(modelColumnIndex, p_asc);
		//  table model fires "Sorted" DataStatus event which causes MTab to position to row 0
	}   //  sort

	/**
	 *  Transfer focus explicitly to editor due to editors with multiple components
	 *
	 *  @param row row
	 *  @param column column
	 *  @param e event
	 *  @return true if cell is editing
	 */
	public boolean editCellAt (int row, int column, java.util.EventObject e)
	{
		if (!super.editCellAt(row, column, e))
			return false;
	//	log.fine( "VTable.editCellAt", "r=" + row + ", c=" + column);

		Object ed = getCellEditor();
		if (ed instanceof VEditor)
			((Component)ed).requestFocus();
		else if (ed instanceof VCellEditor)
		{
			ed = ((VCellEditor)ed).getEditor();
			((Component)ed).requestFocus();
		}
		else if (ed instanceof VFormatedCellEditor)
		{
			ed = ((VFormatedCellEditor)ed).getEditor();
			((Component)ed).requestFocus();
		}
		char tempChar = currentKeyChar;
		return true;
	}   //  editCellAt

	/**
	 *  toString
	 *  @return String representation
	 */
	public String toString()
	{
		return new StringBuffer("VTable[")
			.append(getModel()).append("]").toString();
	}   //  toString
	
}	//	VTable
